@@ -16,21 +16,17 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "ProjectPages.h"
-#include "ProjectPages.moc"
+#include "ProjectPages.h.moc"
 #include "ImageFileInfo.h"
 #include "ImageMetadata.h"
 #include "ImageInfo.h"
 #include "OrthogonalRotation.h"
 #include "PageSequence.h"
-#include "RelinkablePath.h"
-#include "AbstractRelinker.h"
-#ifndef Q_MOC_RUN
+#include <boost/foreach.hpp>
 #include <boost/multi_index_container.hpp>
 #include <boost/multi_index/ordered_index.hpp>
 #include <boost/multi_index/sequenced_index.hpp>
 #include <boost/multi_index/member.hpp>
-#endif
 #include <QMutexLocker>
 #include <QFileInfo>
 #include <QSize>
@@ -51,7 +47,7 @@ ProjectPages::ProjectPages(
 {
 	initSubPagesInOrder(layout_direction);
 	
-	for (ImageInfo const& image: info) {
+	BOOST_FOREACH(ImageInfo const& image, info) {
 		ImageDesc image_desc(image);
 		
 		// Enforce some rules.
@@ -75,7 +71,7 @@ ProjectPages::ProjectPages(
 {
 	initSubPagesInOrder(layout_direction);
 	
-	for (ImageFileInfo const& file: files) {
+	BOOST_FOREACH(ImageFileInfo const& file, files) {
 		QString const& file_path = file.fileInfo().absoluteFilePath();
 		std::vector<ImageMetadata> const& images = file.imageInfo();
 		int const num_images = images.size();
@@ -167,39 +163,6 @@ ProjectPages::toPageSequence(PageView const view) const
 	return pages;
 }
 
-void
-ProjectPages::listRelinkablePaths(VirtualFunction1<void, RelinkablePath const&>& sink) const
-{
-	// It's generally a bad idea to do callbacks while holding an internal mutex,
-	// so we accumulate results into this vector first.
-	std::vector<QString> files;
-	
-	{
-		QMutexLocker locker(&m_mutex);
-		
-		files.reserve(m_images.size());
-		for (ImageDesc const& image: m_images) {
-			files.push_back(image.id.filePath());
-		}
-	}
-
-	for (QString const& file: files) {
-		sink(RelinkablePath(file, RelinkablePath::File));
-	}
-}
-
-void
-ProjectPages::performRelinking(AbstractRelinker const& relinker)
-{
-	QMutexLocker locker(&m_mutex);
-
-	for (ImageDesc& image: m_images) {
-		RelinkablePath const old_path(image.id.filePath(), RelinkablePath::File);
-		QString const new_path(relinker.substitutionPathFor(old_path));
-		image.id.setFilePath(new_path);
-	}
-}
-
 void
 ProjectPages::setLayoutTypeFor(ImageId const& image_id, LayoutType const layout)
 {
@@ -283,32 +246,23 @@ ProjectPages::numImages() const
 	return m_images.size();
 }
 
-int
-ProjectPages::getMaxPageNum(QString const& filepath) const
-{
-    return getMaxPageNumImpl(filepath);
-}
-
 std::vector<PageInfo>
 ProjectPages::insertImage(
 	ImageInfo const& new_image, BeforeOrAfter before_or_after,
 	ImageId const& existing, PageView const view)
 {
 	bool was_modified = false;
-    std::vector<PageInfo> res;
-
+	
 	{
 		QMutexLocker locker(&m_mutex);
-        res = insertImageImpl(
+		return insertImageImpl(
 			new_image, before_or_after, existing, view, was_modified
 		);
 	}
-
-    if (was_modified) {
-        emit modified();
-    }
-
-    return res;
+	
+	if (was_modified) {
+		emit modified();
+	}
 }
 
 void
@@ -350,7 +304,7 @@ ProjectPages::validateDpis() const
 {
 	QMutexLocker locker(&m_mutex);
 	
-	for (ImageDesc const& image: m_images) {
+	BOOST_FOREACH(ImageDesc const& image, m_images) {
 		if (!image.metadata.isDpiOK()) {
 			return false;
 		}
@@ -390,7 +344,7 @@ ProjectPages::toImageFileInfo() const
 	{
 		QMutexLocker locker(&m_mutex);
 		
-		for (ImageDesc const& image: m_images) {
+		BOOST_FOREACH(ImageDesc const& image, m_images) {
 			File const file(image.id.filePath());
 			files.insert(file).first->metadata.push_back(image.metadata);
 		}
@@ -405,10 +359,10 @@ ProjectPages::updateMetadataFrom(std::vector<ImageFileInfo> const& files)
 	typedef std::map<ImageId, ImageMetadata> MetadataMap;
 	MetadataMap metadata_map;
 	
-	for (ImageFileInfo const& file: files) {
+	BOOST_FOREACH(ImageFileInfo const& file, files) {
 		QString const file_path(file.fileInfo().absoluteFilePath());
 		int page = 0;
-		for (ImageMetadata const& metadata: file.imageInfo()) {
+		BOOST_FOREACH(ImageMetadata const& metadata, file.imageInfo()) {
 			metadata_map[ImageId(file_path, page)] = metadata;
 			++page;
 		}
@@ -416,7 +370,7 @@ ProjectPages::updateMetadataFrom(std::vector<ImageFileInfo> const& files)
 	
 	QMutexLocker locker(&m_mutex);
 	
-	for (ImageDesc& image: m_images) {
+	BOOST_FOREACH(ImageDesc& image, m_images) {
 		MetadataMap::const_iterator const it(metadata_map.find(image.id));
 		if (it != metadata_map.end()) {
 			image.metadata = it->second;
@@ -525,27 +479,11 @@ ProjectPages::updateImageMetadataImpl(
 	}
 }
 
-int
-ProjectPages::getMaxPageNumImpl(QString const& filepath) const
-{
-    int max_page_num = -1;
-    std::vector<ImageDesc>::const_iterator it(m_images.cbegin());
-    std::vector<ImageDesc>::const_iterator const end(m_images.cend());
-    for (; it != end; ++it) {
-        const ImageId& image_id = it->id;
-        if (image_id.filePath() == filepath && max_page_num < image_id.page()) {
-            max_page_num = image_id.page();
-        }
-    }
-    return max_page_num;
-}
-
 std::vector<PageInfo>
 ProjectPages::insertImageImpl(
 	ImageInfo const& new_image, BeforeOrAfter before_or_after,
 	ImageId const& existing, PageView const view, bool& modified)
 {	
-    modified = false;
 	std::vector<PageInfo> logical_pages;
 
 	std::vector<ImageDesc>::iterator it(m_images.begin());
@@ -577,7 +515,6 @@ ProjectPages::insertImageImpl(
 	}
 
 	m_images.insert(it, image_desc);
-    modified = true;
 	
 	PageInfo page_info_templ(
 		PageId(new_image.id(), PageId::SINGLE_PAGE),
@@ -617,25 +554,40 @@ ProjectPages::removePagesImpl(std::set<PageId> const& to_remove, bool& modified)
 	for (int i = 0; i < num_old_images; ++i) {
 		ImageDesc image(m_images[i]);
 
-		if (to_remove.find(PageId(image.id, PageId::SINGLE_PAGE)) != to_remove_end) {
-			image.numLogicalPages = 0;
-			modified = true;
+		if (image.numLogicalPages == 1) {
+			if (to_remove.find(PageId(image.id, PageId::SINGLE_PAGE)) == to_remove_end) {
+				new_images.push_back(image);
+				new_total_logical_pages += image.numLogicalPages;
+			} else {
+				modified = true;
+			}
 		} else {
-			if (to_remove.find(PageId(image.id, PageId::LEFT_PAGE)) != to_remove_end) {
-				image.leftHalfRemoved = true;
-				--image.numLogicalPages;
-				modified = true;
-			}
-			if (to_remove.find(PageId(image.id, PageId::RIGHT_PAGE)) != to_remove_end) {
-				image.rightHalfRemoved = true;
-				--image.numLogicalPages;
-				modified = true;
-			}
-		}
+			assert(image.numLogicalPages == 2);
 
-		if (image.numLogicalPages > 0) {
-			new_images.push_back(image);
-			new_total_logical_pages += new_images.back().numLogicalPages;
+			int subpages_to_remove = 0;
+			if (to_remove.find(PageId(image.id, PageId::SINGLE_PAGE)) != to_remove_end) {
+				subpages_to_remove = 2;
+			} else {
+				if (to_remove.find(PageId(image.id, PageId::LEFT_PAGE)) != to_remove_end) {
+					image.leftHalfRemoved = true;
+					--image.numLogicalPages;
+					++subpages_to_remove;
+				}
+				if (to_remove.find(PageId(image.id, PageId::RIGHT_PAGE)) != to_remove_end) {
+					image.rightHalfRemoved = true;
+					--image.numLogicalPages;
+					++subpages_to_remove;
+				}
+			}
+
+			if (subpages_to_remove < 2) {
+				new_images.push_back(image);
+				new_total_logical_pages += new_images.back().numLogicalPages;
+			}
+
+			if (subpages_to_remove > 0) {
+				modified = true;
+			}
 		}
 	}
 
