@@ -32,7 +32,7 @@
 #include <vector>
 
 class PageId;
-class PageSequence;
+class ProjectPages;
 class PageSelectionAccessor;
 
 class StageSequence : public RefCountable
@@ -41,7 +41,7 @@ class StageSequence : public RefCountable
 public:
 	typedef IntrusivePtr<AbstractFilter> FilterPtr;
 	
-	StageSequence(IntrusivePtr<PageSequence> const& pages,
+	StageSequence(IntrusivePtr<ProjectPages> const& pages,
 		PageSelectionAccessor const& page_selection_accessor);
 	
 	std::vector<FilterPtr> const& filters() const { return m_filters; }
