@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include "Dpi.h"
 #include "ColorParams.h"
 #include "OutputParams.h"
+#include "DespeckleLevel.h"
 #include "ZoneSet.h"
 #include "PropertySet.h"
 #include <QMutex>
@@ -61,6 +62,12 @@ public:
 	void setDpi(PageId const& page_id, Dpi const& dpi);
 	
 	void setDpiForAllPages(Dpi const& dpi);
+
+	DespeckleLevel getDespeckleLevel(PageId const& page_id) const;
+
+	void setDespeckleLevel(PageId const& page_id, DespeckleLevel level);
+
+	void setDespeckleLevelForAllPages(DespeckleLevel level);
 	
 	std::auto_ptr<OutputParams> getOutputParams(PageId const& page_id) const;
 	
@@ -68,19 +75,28 @@ public:
 	
 	void setOutputParams(PageId const& page_id, OutputParams const& params);
 
-	ZoneSet zonesForPage(PageId const& page_id) const;
+	ZoneSet pictureZonesForPage(PageId const& page_id) const;
 
-	void setZones(PageId const& page_id, ZoneSet const& zones);
+	ZoneSet fillZonesForPage(PageId const& page_id) const;
+
+	void setPictureZones(PageId const& page_id, ZoneSet const& zones);
+
+	void setFillZones(PageId const& page_id, ZoneSet const& zones);
 
 	/**
 	 * For now, default zone properties are not persistent.
 	 * They may become persistent later though.
 	 */
-	PropertySet defaultZoneProperties() const;
+	PropertySet defaultPictureZoneProperties() const;
 
-	void setDefaultZoneProperties(PropertySet const& props);
+	PropertySet defaultFillZoneProperties() const;
+
+	void setDefaultPictureZoneProperties(PropertySet const& props);
+
+	void setDefaultFillZoneProperties(PropertySet const& props);
 private:
 	typedef std::map<PageId, Dpi> PerPageDpi;
+	typedef std::map<PageId, DespeckleLevel> PerPageDespeckleLevel;
 	typedef std::map<PageId, ColorParams> PerPageColorParams;
 	typedef std::map<PageId, OutputParams> PerPageOutputParams;
 	typedef std::map<PageId, ZoneSet> PerPageZones;
@@ -88,6 +104,10 @@ private:
 	Dpi getDpiLocked(PageId const& page_id) const;
 	
 	void setDpiLocked(PageId const& page_id, Dpi const& dpi);
+
+	DespeckleLevel getDespeckleLevelLocked(PageId const& page_id) const;
+
+	void setDespeckleLevelLocked(PageId const& page_id, DespeckleLevel level);
 	
 	ColorParams getColorParamsLocked(PageId const& page_id) const;
 	
@@ -96,12 +116,16 @@ private:
 	
 	mutable QMutex m_mutex;
 	PerPageDpi m_perPageDpi;
+	PerPageDespeckleLevel m_perPageDespeckleLevel;
 	PerPageColorParams m_perPageColorParams;
 	PerPageOutputParams m_perPageOutputParams;
-	PerPageZones m_perPageZones;
+	PerPageZones m_perPagePictureZones;
+	PerPageZones m_perPageFillZones;
 	Dpi m_defaultDpi;
 	ColorParams m_defaultColorParams;
-	PropertySet m_defaultZoneProps;
+	PropertySet m_defaultPictureZoneProps;
+	PropertySet m_defaultFillZoneProps;
+	DespeckleLevel m_defaultDespeckleLevel;
 };
 
 } // namespace output
