@@ -50,8 +50,7 @@ CacheDrivenTask::~CacheDrivenTask()
 
 void
 CacheDrivenTask::process(
-	PageInfo const& page_info, int const page_num,
-	AbstractFilterDataCollector* collector,
+	PageInfo const& page_info, AbstractFilterDataCollector* collector,
 	ImageTransformation const& xform, QRectF const& content_rect)
 {
 	std::auto_ptr<Params> const params(
@@ -87,7 +86,7 @@ CacheDrivenTask::process(
 			)
 		);
 		m_ptrNextTask->process(
-			page_info, page_num, collector, xform,
+			page_info, collector, xform,
 			content_rect_phys, page_rect_phys
 		);
 		return;
