@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "ImageView.h"
 #include "ImageView.h.moc"
 #include "OptionsWidget.h"
 #include "Margins.h"
@@ -245,9 +246,17 @@ void
 ImageView::alignmentChanged(Alignment const& alignment)
 {
 	m_alignment = alignment;
-	m_ptrSettings->setPageAlignment(m_pageId, alignment);
+
+	Settings::AggregateSizeChanged const size_changed =
+		m_ptrSettings->setPageAlignment(m_pageId, alignment);
+	
 	recalcBoxesAndFit(calcHardMarginsMM());
-	emit invalidateThumbnail(m_pageId);
+	
+	if (size_changed == Settings::AGGREGATE_SIZE_CHANGED) {
+		emit invalidateAllThumbnails();
+	} else {
+		emit invalidateThumbnail(m_pageId);
+	}
 }
 
 void
@@ -262,6 +271,19 @@ ImageView::aggregateHardSizeChanged()
 void
 ImageView::onPaint(QPainter& painter, InteractionState const& interaction)
 {	
+	QColor bg_color;
+	QColor fg_color;
+	if (m_alignment.isNull()) {
+		// "Align with other pages" is turned off.
+		// Different color is useful on a thumbnail list to
+		// distinguish "safe" pages from potentially problematic ones.
+		bg_color = QColor(0x58, 0x7f, 0xf4, 70);
+		fg_color = QColor(0x00, 0x52, 0xff);
+	} else {
+		bg_color = QColor(0xbb, 0x00, 0xff, 40);
+		fg_color = QColor(0xbe, 0x5b, 0xec);
+	}
+
 	QPainterPath outer_outline;
 	outer_outline.addPolygon(
 		PolygonUtils::round(
@@ -275,10 +297,10 @@ ImageView::onPaint(QPainter& painter, InteractionState const& interaction)
 	painter.setRenderHint(QPainter::Antialiasing, false);
 	
 	painter.setPen(Qt::NoPen);
-	painter.setBrush(QColor(0xbb, 0x00, 0xff, 40));
+	painter.setBrush(bg_color);
 	painter.drawPath(outer_outline.subtracted(content_outline));
 	
-	QPen pen(QColor(0xbe, 0x5b, 0xec));
+	QPen pen(fg_color);
 	pen.setCosmetic(true);
 	pen.setWidthF(2.0);
 	painter.setPen(pen);
@@ -440,7 +462,7 @@ ImageView::middleRectDragContinuation(int const edge_mask, QPointF const& mouse_
 	qreal top_adjust = 0;
 	qreal bottom_adjust = 0;
 
-	QRectF const bounds(viewportRect());
+	QRectF const bounds(maxViewportRect());
 	QRectF const old_middle_rect(m_beforeResizing.middleWidgetRect);
 
 	if (edge_mask & LEFT) {
@@ -504,7 +526,7 @@ ImageView::dragFinished()
 		commitHardMargins(calcHardMarginsMM())
 	);
 
-	QRectF const extended_viewport(viewportRect().adjusted(-0.5, -0.5, 0.5, 0.5));
+	QRectF const extended_viewport(maxViewportRect().adjusted(-0.5, -0.5, 0.5, 0.5));
 	if (extended_viewport.contains(m_beforeResizing.middleWidgetRect)) {
 		updatePresentationTransform(FIT);
 	} else {
