@@ -114,9 +114,13 @@ ContentBoxFinder::findContentBox(
 {
 	ImageTransformation xform_150dpi(data.xform());
 	xform_150dpi.preScaleToDpi(Dpi(150, 150));
+
+	if (xform_150dpi.resultingRect().toRect().isEmpty()) {
+		return QRectF();
+	}
 	
 	uint8_t const darkest_gray_level = darkestGrayLevel(data.grayImage());
-	
+
 	QImage gray150(
 		transformToGray(
 			data.grayImage(), xform_150dpi.transform(),
@@ -131,7 +135,7 @@ ContentBoxFinder::findContentBox(
 		dbg->add(gray150, "gray150");
 	}
 	
-	BinaryImage bw150(binarizeWolf(gray150, QSize(51, 51), 80));
+	BinaryImage bw150(binarizeWolf(gray150, QSize(51, 51), 50));
 	if (dbg) {
 		dbg->add(bw150, "bw150");
 	}
@@ -204,8 +208,7 @@ ContentBoxFinder::findContentBox(
 	
 	status.throwIfCancelled();
 	
-	int const big_object_threshold = 25;
-	BinaryImage despeckled(despeckle(content, big_object_threshold));
+	BinaryImage despeckled(Despeckle::despeckle(content, Dpi(150, 150), Despeckle::NORMAL, status));
 	if (dbg) {
 		dbg->add(despeckled, "despeckled");
 	}
