@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C) 2007-2008  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,21 +26,33 @@
 #include "ProjectReader.h"
 #include "ProjectWriter.h"
 #include "CacheDrivenTask.h"
+#include "OrderByWidthProvider.h"
+#include "OrderByHeightProvider.h"
 #include <boost/lambda/lambda.hpp>
 #include <boost/lambda/bind.hpp>
 #include <QString>
 #include <QObject>
-#include <QCoreApplication>
 #include <QDomDocument>
 #include <QDomElement>
+#include <assert.h>
 
 namespace select_content
 {
 
 Filter::Filter()
-:	m_ptrSettings(new Settings)
+:	m_ptrSettings(new Settings),
+	m_selectedPageOrder(0)
 {
 	m_ptrOptionsWidget.reset(new OptionsWidget(m_ptrSettings));
+
+	typedef PageOrderOption::ProviderPtr ProviderPtr;
+
+	ProviderPtr const default_order;
+	ProviderPtr const order_by_width(new OrderByWidthProvider(m_ptrSettings));
+	ProviderPtr const order_by_height(new OrderByHeightProvider(m_ptrSettings));
+	m_pageOrderOptions.push_back(PageOrderOption(tr("Natural order"), default_order));
+	m_pageOrderOptions.push_back(PageOrderOption(tr("Order by increasing width"), order_by_width));
+	m_pageOrderOptions.push_back(PageOrderOption(tr("Order by increasing height"), order_by_height));
 }
 
 Filter::~Filter()
@@ -50,15 +62,32 @@ Filter::~Filter()
 QString
 Filter::getName() const
 {
-	return QCoreApplication::translate(
-		"select_content::Filter", "Select Content"
-	);
+	return tr("Select Content");
 }
 
-PageSequence::View
+PageView
 Filter::getView() const
 {
-	return PageSequence::PAGE_VIEW;
+	return PAGE_VIEW;
+}
+
+int
+Filter::selectedPageOrder() const
+{
+	return m_selectedPageOrder;
+}
+
+void
+Filter::selectPageOrder(int option)
+{
+	assert((unsigned)option < m_pageOrderOptions.size());
+	m_selectedPageOrder = option;
+}
+
+std::vector<PageOrderOption>
+Filter::pageOrderOptions() const
+{
+	return m_pageOrderOptions;
 }
 
 void
