@@ -1,6 +1,6 @@
 /*
     Scan Tailor - Interactive post-processing tool for scanned pages.
-    Copyright (C) 2007-2008  Joseph Artsimovich <joseph_a@mail.ru>
+    Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "OptionsWidget.h"
 #include "OptionsWidget.h.moc"
 #include "Settings.h"
 #include "Params.h"
@@ -101,7 +102,7 @@ void
 OptionsWidget::commitCurrentParams()
 {
 	Params const params(
-		m_uiData.contentRect(),
+		m_uiData.contentRect(), m_uiData.contentSizeMM(),
 		m_uiData.dependencies(), m_uiData.mode()
 	);
 	m_ptrSettings->setPageParams(m_pageId, params);
@@ -119,6 +120,12 @@ OptionsWidget::UiData::~UiData()
 {
 }
 
+void
+OptionsWidget::UiData::setSizeCalc(PhysSizeCalc const& calc)
+{
+	m_sizeCalc = calc;
+}
+
 void
 OptionsWidget::UiData::setContentRect(QRectF const& content_rect)
 {
@@ -131,6 +138,12 @@ OptionsWidget::UiData::contentRect() const
 	return m_contentRect;
 }
 
+QSizeF
+OptionsWidget::UiData::contentSizeMM() const
+{
+	return m_sizeCalc.sizeMM(m_contentRect);
+}
+
 void
 OptionsWidget::UiData::setDependencies(Dependencies const& deps)
 {
