@@ -1,6 +1,6 @@
 /*
 	Scan Tailor - Interactive post-processing tool for scanned pages.
-	Copyright (C) 2007-2009  Joseph Artsimovich <joseph_a@mail.ru>
+	Copyright (C)  Joseph Artsimovich <joseph.artsimovich@gmail.com>
 
 	This program is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include <QPolygonF>
 
 class EditableSpline;
+class QTransform;
 class QDomDocument;
 class QDomElement;
 class QString;
@@ -37,6 +38,8 @@ public:
 
 	QDomElement toXml(QDomDocument& doc, QString const& name) const;
 
+	SerializableSpline transformed(QTransform const& xform) const;
+
 	QPolygonF toPolygon() const { return QPolygonF(m_points); }
 private:
 	QVector<QPointF> m_points;
